@@ -1,10 +1,9 @@
-# -*- coding: utf-8 -*-
 """Calendar is a dictionary like Python object that can render itself as VCAL
 files according to rfc2445.
 
 These are the defined components.
 """
-from datetime import datetime
+from datetime import datetime, timedelta
 from icalendar.caselessdict import CaselessDict
 from icalendar.parser import Contentline
 from icalendar.parser import Contentlines
@@ -12,11 +11,14 @@ from icalendar.parser import Parameters
 from icalendar.parser import q_join
 from icalendar.parser import q_split
 from icalendar.parser_tools import DEFAULT_ENCODING
-from icalendar.parser_tools import data_encode
 from icalendar.prop import TypesFactory
 from icalendar.prop import vText, vDDDLists
+from icalendar.timezone_cache import _timezone_cache
 
 import pytz
+import dateutil.rrule, dateutil.tz
+from pytz.tzinfo import DstTzInfo
+
 
 
 ######################################
@@ -30,7 +32,7 @@ class ComponentFactory(CaselessDict):
     def __init__(self, *args, **kwargs):
         """Set keys to upper for initial dict.
         """
-        super(ComponentFactory, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self['VEVENT'] = Event
         self['VTODO'] = Todo
         self['VJOURNAL'] = Journal
@@ -56,41 +58,58 @@ _marker = []
 class Component(CaselessDict):
     """Component is the base object for calendar, Event and the other
     components defined in RFC 2445. normally you will not use this class
-    directy, but rather one of the subclasses.
+    directly, but rather one of the subclasses.
     """
 
-    name = ''        # must be defined in each component
-    required = ()    # These properties are required
-    singletons = ()  # These properties must only appear once
-    multiple = ()    # may occur more than once
-    exclusive = ()   # These properties are mutually exclusive
-    inclusive = ()   # if any occurs the other(s) MUST occur
-                     # ('duration', 'repeat')
+    name = None         # should be defined in each component
+    required = ()       # These properties are required
+    singletons = ()     # These properties must only appear once
+    multiple = ()       # may occur more than once
+    exclusive = ()      # These properties are mutually exclusive
+    inclusive = ()      # if any occurs the other(s) MUST occur
+                        # ('duration', 'repeat')
     ignore_exceptions = False   # if True, and we cannot parse this
                                 # component, we will silently ignore
                                 # it, rather than let the exception
                                 # propagate upwards
-    # not_compliant = [''] # List of non-compliant properties.
+    # not_compliant = ['']  # List of non-compliant properties.
 
     def __init__(self, *args, **kwargs):
         """Set keys to upper for initial dict.
         """
-        super(Component, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         # set parameters here for properties that use non-default values
         self.subcomponents = []  # Components can be nested.
-        self.is_broken = False  # True if we ignored an exception while
-                                # parsing a property
+        self.errors = []  # If we ignored exception(s) while
+                          # parsing a property, contains error strings
 
-    #def is_compliant(self, name):
+    # def is_compliant(self, name):
     #    """Returns True is the given property name is compliant with the
     #    icalendar implementation.
     #
     #    If the parser is too strict it might prevent parsing erroneous but
     #    otherwise compliant properties. So the parser is pretty lax, but it is
-    #    possible to test for non-complience by calling this method.
+    #    possible to test for non-compliance by calling this method.
     #    """
     #    return name in not_compliant
 
+    def __bool__(self):
+        """Returns True, CaselessDict would return False if it had no items.
+        """
+        return True
+
+    # python 2 compatibility
+    __nonzero__ = __bool__
+
+    def is_empty(self):
+        """Returns True if Component has no items or subcomponents, else False.
+        """
+        return True if not (list(self.values()) + self.subcomponents) else False  # noqa
+
+    @property
+    def is_broken(self):
+        return bool(self.errors)
+
     #############################
     # handling of property values
 
@@ -164,7 +183,7 @@ class Component(CaselessDict):
 
         # encode value
         if encode and isinstance(value, list) \
-                and name.lower() not in ['rdate', 'exdate']:
+                and name.lower() not in ['rdate', 'exdate', 'categories']:
             # Individually convert each value to an ical type except rdate and
             # exdate, where lists of dates might be passed to vDDDLists.
             value = [self._encode(name, v, parameters, encode) for v in value]
@@ -233,7 +252,7 @@ class Component(CaselessDict):
         return vals
 
     def set_inline(self, name, values, encode=1):
-        """Converts a list of values into comma seperated string and sets value
+        """Converts a list of values into comma separated string and sets value
         to that.
         """
         if encode:
@@ -262,7 +281,7 @@ class Component(CaselessDict):
         """Recursively traverses component and subcomponents. Returns sequence
         of same. If name is passed, only components with name will be returned.
         """
-        if not name is None:
+        if name is not None:
             name = name.upper()
         return self._walk(name)
 
@@ -307,14 +326,14 @@ class Component(CaselessDict):
 
             try:
                 name, params, vals = line.parts()
-            except ValueError:
+            except ValueError as e:
                 # if unable to parse a line within a component
                 # that ignores exceptions, mark the component
                 # as broken and skip the line. otherwise raise.
                 component = stack[-1] if stack else None
                 if not component or not component.ignore_exceptions:
                     raise
-                component.is_broken = True
+                component.errors.append((None, str(e)))
                 continue
 
             uname = name.upper()
@@ -323,7 +342,11 @@ class Component(CaselessDict):
                 # try and create one of the components defined in the spec,
                 # otherwise get a general Components for robustness.
                 c_name = vals.upper()
-                c_class = component_factory.get(c_name, cls)
+                c_class = component_factory.get(c_name, Component)
+                # If component factory cannot resolve ``c_name``, the generic
+                # ``Component`` class is used which does not have the name set.
+                # That's opposed to the usage of ``cls``, which represents a
+                # more concrete subclass with a name set (e.g. VCALENDAR).
                 component = c_class()
                 if not getattr(component, 'name', ''):  # undefined components
                     component.name = c_name
@@ -336,12 +359,19 @@ class Component(CaselessDict):
                 if not stack:  # we are at the end
                     comps.append(component)
                 else:
-                    if not component.is_broken:
-                        stack[-1].add_component(component)
+                    stack[-1].add_component(component)
+                if vals == 'VTIMEZONE' and \
+                        'TZID' in component and \
+                        component['TZID'] not in pytz.all_timezones and \
+                        component['TZID'] not in _timezone_cache:
+                    _timezone_cache[component['TZID']] = component.to_tz()
             # we are adding properties to the current top of the stack
             else:
                 factory = types_factory.for_property(name)
-                component = stack[-1]
+                component = stack[-1] if stack else None
+                if not component:
+                    raise ValueError('Property "{prop}" does not have '
+                                     'a parent component.'.format(prop=name))
                 datetime_names = ('DTSTART', 'DTEND', 'RECURRENCE-ID', 'DUE',
                                   'FREEBUSY', 'RDATE', 'EXDATE')
                 try:
@@ -349,10 +379,11 @@ class Component(CaselessDict):
                         vals = factory(factory.from_ical(vals, params['TZID']))
                     else:
                         vals = factory(factory.from_ical(vals))
-                except ValueError:
+                except ValueError as e:
                     if not component.ignore_exceptions:
                         raise
-                    component.is_broken = True
+                    component.errors.append((uname, str(e)))
+                    component.add(name, None, encode=0)
                 else:
                     vals.params = params
                     component.add(name, vals, encode=0)
@@ -360,17 +391,14 @@ class Component(CaselessDict):
         if multiple:
             return comps
         if len(comps) > 1:
-            raise ValueError('Found multiple components where '
-                             'only one is allowed: {st!r}'.format(**locals()))
+            raise ValueError(f'Found multiple components where '
+                             f'only one is allowed: {st!r}')
         if len(comps) < 1:
-            raise ValueError('Found no components where '
-                             'exactly one is required: '
-                             '{st!r}'.format(**locals()))
+            raise ValueError(f'Found no components where '
+                             f'exactly one is required: '
+                             f'{st!r}')
         return comps[0]
 
-    def __repr__(self):
-        return '%s(%s)' % (self.name, data_encode(self))
-
     def content_line(self, name, value, sorted=True):
         """Returns property as content line.
         """
@@ -390,15 +418,25 @@ class Component(CaselessDict):
     def to_ical(self, sorted=True):
         '''
         :param sorted: Whether parameters and properties should be
-        lexicographically sorted.
+                       lexicographically sorted.
         '''
 
         content_lines = self.content_lines(sorted=sorted)
         return content_lines.to_ical()
 
+    def __repr__(self):
+        """String representation of class with all of it's subcomponents.
+        """
+        subs = ', '.join([str(it) for it in self.subcomponents])
+        return '{}({}{})'.format(
+            self.name or type(self).__name__,
+            dict(self),
+            ', %s' % subs if subs else ''
+        )
+
 
 #######################################
-# components defined in RFC 2445
+# components defined in RFC 5545
 
 class Event(Component):
 
@@ -406,21 +444,21 @@ class Event(Component):
 
     canonical_order = (
         'SUMMARY', 'DTSTART', 'DTEND', 'DURATION', 'DTSTAMP',
-        'UID', 'RECURRENCE-ID', 'SEQUENCE',
-        'RRULE' 'EXRULE', 'RDATE', 'EXDATE',
+        'UID', 'RECURRENCE-ID', 'SEQUENCE', 'RRULE', 'RDATE',
+        'EXDATE',
     )
 
-    required = ('UID',)
+    required = ('UID', 'DTSTAMP',)
     singletons = (
         'CLASS', 'CREATED', 'DESCRIPTION', 'DTSTART', 'GEO', 'LAST-MODIFIED',
         'LOCATION', 'ORGANIZER', 'PRIORITY', 'DTSTAMP', 'SEQUENCE', 'STATUS',
         'SUMMARY', 'TRANSP', 'URL', 'RECURRENCE-ID', 'DTEND', 'DURATION',
-        'DTSTART',
+        'UID', 'CATEGORIES',
     )
-    exclusive = ('DTEND', 'DURATION', )
+    exclusive = ('DTEND', 'DURATION',)
     multiple = (
-        'ATTACH', 'ATTENDEE', 'CATEGORIES', 'COMMENT', 'CONTACT', 'EXDATE',
-        'EXRULE', 'RSTATUS', 'RELATED', 'RESOURCES', 'RDATE', 'RRULE'
+        'ATTACH', 'ATTENDEE', 'COMMENT', 'CONTACT', 'EXDATE',
+        'RSTATUS', 'RELATED', 'RESOURCES', 'RDATE', 'RRULE'
     )
     ignore_exceptions = True
 
@@ -429,17 +467,17 @@ class Todo(Component):
 
     name = 'VTODO'
 
-    required = ('UID',)
+    required = ('UID', 'DTSTAMP',)
     singletons = (
         'CLASS', 'COMPLETED', 'CREATED', 'DESCRIPTION', 'DTSTAMP', 'DTSTART',
-        'GEO', 'LAST-MODIFIED', 'LOCATION', 'ORGANIZER', 'PERCENT', 'PRIORITY',
-        'RECURRENCE-ID', 'SEQUENCE', 'STATUS', 'SUMMARY', 'UID', 'URL', 'DUE',
-        'DURATION',
+        'GEO', 'LAST-MODIFIED', 'LOCATION', 'ORGANIZER', 'PERCENT-COMPLETE',
+        'PRIORITY', 'RECURRENCE-ID', 'SEQUENCE', 'STATUS', 'SUMMARY', 'UID',
+        'URL', 'DUE', 'DURATION',
     )
     exclusive = ('DUE', 'DURATION',)
     multiple = (
         'ATTACH', 'ATTENDEE', 'CATEGORIES', 'COMMENT', 'CONTACT', 'EXDATE',
-        'EXRULE', 'RSTATUS', 'RELATED', 'RESOURCES', 'RDATE', 'RRULE'
+        'RSTATUS', 'RELATED', 'RESOURCES', 'RDATE', 'RRULE'
     )
 
 
@@ -447,15 +485,14 @@ class Journal(Component):
 
     name = 'VJOURNAL'
 
-    required = ('UID',)
+    required = ('UID', 'DTSTAMP',)
     singletons = (
-        'CLASS', 'CREATED', 'DESCRIPTION', 'DTSTART', 'DTSTAMP',
-        'LAST-MODIFIED', 'ORGANIZER', 'RECURRENCE-ID', 'SEQUENCE', 'STATUS',
-        'SUMMARY', 'UID', 'URL',
+        'CLASS', 'CREATED', 'DTSTART', 'DTSTAMP', 'LAST-MODIFIED', 'ORGANIZER',
+        'RECURRENCE-ID', 'SEQUENCE', 'STATUS', 'SUMMARY', 'UID', 'URL',
     )
     multiple = (
         'ATTACH', 'ATTENDEE', 'CATEGORIES', 'COMMENT', 'CONTACT', 'EXDATE',
-        'EXRULE', 'RELATED', 'RDATE', 'RRULE', 'RSTATUS',
+        'RELATED', 'RDATE', 'RRULE', 'RSTATUS', 'DESCRIPTION',
     )
 
 
@@ -463,9 +500,9 @@ class FreeBusy(Component):
 
     name = 'VFREEBUSY'
 
-    required = ('UID',)
+    required = ('UID', 'DTSTAMP',)
     singletons = (
-        'CONTACT', 'DTSTART', 'DTEND', 'DURATION', 'DTSTAMP', 'ORGANIZER',
+        'CONTACT', 'DTSTART', 'DTEND', 'DTSTAMP', 'ORGANIZER',
         'UID', 'URL',
     )
     multiple = ('ATTENDEE', 'COMMENT', 'FREEBUSY', 'RSTATUS',)
@@ -473,32 +510,178 @@ class FreeBusy(Component):
 
 class Timezone(Component):
     name = 'VTIMEZONE'
-    canonical_order = ('TZID', 'STANDARD', 'DAYLIGHT',)
-    required = ('TZID', 'STANDARD', 'DAYLIGHT',)
+    canonical_order = ('TZID',)
+    required = ('TZID',) # it also requires one of components DAYLIGHT and STANDARD
     singletons = ('TZID', 'LAST-MODIFIED', 'TZURL',)
 
+    @staticmethod
+    def _extract_offsets(component, tzname):
+        """extract offsets and transition times from a VTIMEZONE component
+        :param component: a STANDARD or DAYLIGHT component
+        :param tzname: the name of the zone
+        """
+        offsetfrom = component['TZOFFSETFROM'].td
+        offsetto = component['TZOFFSETTO'].td
+        dtstart = component['DTSTART'].dt
+
+        # offsets need to be rounded to the next minute, we might loose up
+        # to 30 seconds accuracy, but it can't be helped (datetime
+        # supposedly cannot handle smaller offsets)
+        offsetto_s = int((offsetto.seconds + 30) / 60) * 60
+        offsetto = timedelta(days=offsetto.days, seconds=offsetto_s)
+        offsetfrom_s = int((offsetfrom.seconds + 30) / 60) * 60
+        offsetfrom = timedelta(days=offsetfrom.days, seconds=offsetfrom_s)
+
+        # expand recurrences
+        if 'RRULE' in component:
+            # to be paranoid about correct weekdays
+            # evaluate the rrule with the current offset
+            tzi = dateutil.tz.tzoffset ("(offsetfrom)", offsetfrom)
+            rrstart = dtstart.replace (tzinfo=tzi)
+
+            rrulestr = component['RRULE'].to_ical().decode('utf-8')
+            rrule = dateutil.rrule.rrulestr(rrulestr, dtstart=rrstart)
+            if not {'UNTIL', 'COUNT'}.intersection(component['RRULE'].keys()):
+                # pytz.timezones don't know any transition dates after 2038
+                # either
+                rrule._until = datetime(2038, 12, 31, tzinfo=pytz.UTC)
+
+            # constructing the pytz-timezone requires UTC transition times.
+            # here we construct local times without tzinfo, the offset to UTC
+            # gets subtracted in to_tz().
+            transtimes = [dt.replace (tzinfo=None) for dt in rrule]
+
+        # or rdates
+        elif 'RDATE' in component:
+            if not isinstance(component['RDATE'], list):
+                rdates = [component['RDATE']]
+            else:
+                rdates = component['RDATE']
+            transtimes = [dtstart] + [leaf.dt for tree in rdates for
+                                      leaf in tree.dts]
+        else:
+            transtimes = [dtstart]
+
+        transitions = [(transtime, offsetfrom, offsetto, tzname) for
+                       transtime in set(transtimes)]
+
+        if component.name == 'STANDARD':
+            is_dst = 0
+        elif component.name == 'DAYLIGHT':
+            is_dst = 1
+        return is_dst, transitions
+
+    @staticmethod
+    def _make_unique_tzname(tzname, tznames):
+        """
+        :param tzname: Candidate tzname
+        :param tznames: Other tznames
+        """
+        # TODO better way of making sure tznames are unique
+        while tzname in tznames:
+            tzname += '_1'
+        tznames.add(tzname)
+        return tzname
+
+    def to_tz(self):
+        """convert this VTIMEZONE component to a pytz.timezone object
+        """
+        try:
+            zone = str(self['TZID'])
+        except UnicodeEncodeError:
+            zone = self['TZID'].encode('ascii', 'replace')
+        transitions = []
+        dst = {}
+        tznames = set()
+        for component in self.walk():
+            if type(component) == Timezone:
+                continue
+            assert isinstance(component['DTSTART'].dt, datetime), (
+                "VTIMEZONEs sub-components' DTSTART must be of type datetime, not date"
+            )
+            try:
+                tzname = str(component['TZNAME'])
+            except UnicodeEncodeError:
+                tzname = component['TZNAME'].encode('ascii', 'replace')
+                tzname = self._make_unique_tzname(tzname, tznames)
+            except KeyError:
+                tzname = '{}_{}_{}_{}'.format(
+                    zone,
+                    component['DTSTART'].to_ical().decode('utf-8'),
+                    component['TZOFFSETFROM'].to_ical(),  # for whatever reason this is str/unicode
+                    component['TZOFFSETTO'].to_ical(),  # for whatever reason this is str/unicode
+                )
+                tzname = self._make_unique_tzname(tzname, tznames)
+
+            dst[tzname], component_transitions = self._extract_offsets(
+                component, tzname
+            )
+            transitions.extend(component_transitions)
+
+        transitions.sort()
+        transition_times = [
+            transtime - osfrom for transtime, osfrom, _, _ in transitions
+        ]
+
+        # transition_info is a list with tuples in the format
+        # (utcoffset, dstoffset, name)
+        # dstoffset = 0, if current transition is to standard time
+        #           = this_utcoffset - prev_standard_utcoffset, otherwise
+        transition_info = []
+        for num, (transtime, osfrom, osto, name) in enumerate(transitions):
+            dst_offset = False
+            if not dst[name]:
+                dst_offset = timedelta(seconds=0)
+            else:
+                # go back in time until we find a transition to dst
+                for index in range(num - 1, -1, -1):
+                    if not dst[transitions[index][3]]:  # [3] is the name
+                        dst_offset = osto - transitions[index][2]  # [2] is osto  # noqa
+                        break
+                # when the first transition is to dst, we didn't find anything
+                # in the past, so we have to look into the future
+                if not dst_offset:
+                    for index in range(num, len(transitions)):
+                        if not dst[transitions[index][3]]:  # [3] is the name
+                            dst_offset = osto - transitions[index][2]  # [2] is osto  # noqa
+                            break
+            assert dst_offset is not False
+            transition_info.append((osto, dst_offset, name))
+
+        cls = type(zone, (DstTzInfo,), {
+            'zone': zone,
+            '_utc_transition_times': transition_times,
+            '_transition_info': transition_info
+        })
+
+        return cls()
+
 
 class TimezoneStandard(Component):
     name = 'STANDARD'
     required = ('DTSTART', 'TZOFFSETTO', 'TZOFFSETFROM')
-    singletons = ('DTSTART', 'TZOFFSETTO', 'TZOFFSETFROM', 'RRULE')
-    multiple = ('COMMENT', 'RDATE', 'TZNAME')
+    singletons = ('DTSTART', 'TZOFFSETTO', 'TZOFFSETFROM',)
+    multiple = ('COMMENT', 'RDATE', 'TZNAME', 'RRULE', 'EXDATE')
 
 
 class TimezoneDaylight(Component):
     name = 'DAYLIGHT'
-    required = ('DTSTART', 'TZOFFSETTO', 'TZOFFSETFROM')
-    singletons = ('DTSTART', 'TZOFFSETTO', 'TZOFFSETFROM', 'RRULE')
-    multiple = ('COMMENT', 'RDATE', 'TZNAME')
+    required = TimezoneStandard.required
+    singletons = TimezoneStandard.singletons
+    multiple = TimezoneStandard.multiple
 
 
 class Alarm(Component):
 
     name = 'VALARM'
-    # not quite sure about these ...
+    # some properties MAY/MUST/MUST NOT appear depending on ACTION value
     required = ('ACTION', 'TRIGGER',)
-    singletons = ('ATTACH', 'ACTION', 'TRIGGER', 'DURATION', 'REPEAT',)
-    inclusive = (('DURATION', 'REPEAT',),)
+    singletons = (
+            'ATTACH', 'ACTION', 'DESCRIPTION', 'SUMMARY', 'TRIGGER',
+            'DURATION', 'REPEAT',
+            )
+    inclusive = (('DURATION', 'REPEAT',), ('SUMMARY', 'ATTENDEE',))
+    multiple = ('ATTENDEE', 'ATTACH')
 
 
 class Calendar(Component):
@@ -506,9 +689,8 @@ class Calendar(Component):
     """
     name = 'VCALENDAR'
     canonical_order = ('VERSION', 'PRODID', 'CALSCALE', 'METHOD',)
-    required = ('prodid', 'version', )
-    singletons = ('prodid', 'version', )
-    multiple = ('calscale', 'method', )
+    required = ('PRODID', 'VERSION', )
+    singletons = ('PRODID', 'VERSION', 'CALSCALE', 'METHOD')
 
 # These are read only singleton, so one instance is enough for the module
 types_factory = TypesFactory()
